import struct
import array
import time
import re

class Vec2f:
	def __init__(self, x, y):
		self.x = x
		self.y = y
		self.struct = struct.Struct("ff")
		
	def write(self, f):
		#f.write(struct.pack("f", self.x))
		#f.write(struct.pack("f", self.y))
		f.write(self.struct.pack(self.x, self.y))
		

class Vec3f:
	def __init__(self, x, y, z):
		self.x = x
		self.y = y
		self.z = z
		self.struct = struct.Struct("fff")
		
	def write(self, f):
		#f.write(struct.pack("f", self.x))
		#f.write(struct.pack("f", self.y))
		#f.write(struct.pack("f", self.z))
		f.write(self.struct.pack(self.x, self.y, self.z))

class Triangle:
	def __init__(self, vertex_indices, uv_indices, tri_mat_index):
		self.vertex_indices = vertex_indices
		self.uv_indices = uv_indices
		self.tri_mat_index = tri_mat_index
		self.struct = struct.Struct("LLLLLLL")
		
	def write(self, f):
		#f.write(struct.pack("L", self.vertex_indices[0]))
		#f.write(struct.pack("L", self.vertex_indices[1]))
		#f.write(struct.pack("L", self.vertex_indices[2]))
		
		#f.write(struct.pack("L", self.uv_indices[0]))
		#f.write(struct.pack("L", self.uv_indices[1]))
		#f.write(struct.pack("L", self.uv_indices[2]))
		
		#f.write(struct.pack("L", self.tri_mat_index))
		f.write(self.struct.pack(self.vertex_indices[0], self.vertex_indices[1], self.vertex_indices[2], self.uv_indices[0], self.uv_indices[1], self.uv_indices[2], self.tri_mat_index))

	
class UVSetExposition:
	def __init__(self, uv_set_name, uv_set_index):
		self.uv_set_name = uv_set_name
		self.uv_set_index = uv_set_index
		self.struct = struct.Struct("L")
		
	def write(self, file):
		self.uv_set_name.write(file)
		file.write(self.struct.pack(self.uv_set_index))
		
		
class IndigoMeshString:
	def __init__(self, s):
		self.s = s
		
	def write(self, file):
		# Write length of string
		file.write(struct.pack("L", len(self.s)))
		
		# Write string data
		#for x in self.s:
		#	print("character: " + x)
		#	file.write(struct.pack("c", chr(x)))
		
		file.write(struct.pack(str(len(self.s)) + "s", self.s))
		
	
	
class OldIndigoMesh:

	def __init__(self):
		self.num_uv_mappings = 0
		self.used_materials = [] # IndigoMeshString list
		self.uv_set_expositions = [] # UVSetExposition list
		self.vert_positions = [] # Vec3f list
		self.vert_normals = [] # Vec2f list
		self.uv_pairs = [] # Vec2f list
		self.triangles = [] # Triangle
		
		
	def writeVector(self, file, v):
		# Write length of vector
		file.write(struct.pack("L", len(v)))
		
		# Write vector elements
		for x in v:
			x.write(file)
			
			
	def writeArray(self, file, a, a_num_elems):
		file.write(struct.pack("L", a_num_elems))
		a.tofile(file)
		
	def writeToDisk(self, path):
		f = file(path, 'wb')
		
		MAGIC_NUMBER = 5456751
		FORMAT_VERSION = 1
		
		f.write(struct.pack("L", MAGIC_NUMBER))
		f.write(struct.pack("L", FORMAT_VERSION))
		f.write(struct.pack("L", self.num_uv_mappings))
		
		self.writeVector(f, self.used_materials)
		self.writeVector(f, self.uv_set_expositions)
		self.writeVector(f, self.vert_positions)
		self.writeVector(f, self.vert_normals)
		self.writeVector(f, self.uv_pairs)
		self.writeVector(f, self.triangles)
		
		f.close
		
		

class IndigoMesh:

	def __init__(self):
		self.num_uv_mappings = 0
		self.used_materials = [] # IndigoMeshString list
		self.uv_set_expositions = [] # UVSetExposition list
		
		self.vert_positions = array.array('f') # Vec3f list
		self.vert_normals = array.array('f') # Vec2f list
		self.uv_pairs = array.array('f') # Vec2f list
		self.triangles = array.array('L') # Triangle
		
	def setNumUVMappings(self, x):
		self.num_uv_mappings = x
		
	def addUsedMaterial(self, matname):
		self.used_materials.append(IndigoMeshString(matname))
		
	def addUVSetExposition(self, uv_set_name, uv_set_index):
		self.uv_set_expositions.append(UVSetExposition(IndigoMeshString(uv_set_name), uv_set_index))
		
	def addVertPosition(self, x, y, z):
		self.vert_positions.append(x)
		self.vert_positions.append(y)
		self.vert_positions.append(z)
		
	def addVertNormal(self, x, y, z):
		self.vert_normals.append(x)
		self.vert_normals.append(y)
		self.vert_normals.append(z)
		
	def addUVPair(self, x, y):
		self.uv_pairs.append(x)
		self.uv_pairs.append(y)
		
	def addTriangle(self, v0, v1, v2, uv0, uv1, uv2, mat_index):
		self.triangles.append(v0)
		self.triangles.append(v1)
		self.triangles.append(v2)
		self.triangles.append(uv0)
		self.triangles.append(uv1)
		self.triangles.append(uv2)
		self.triangles.append(mat_index)
		
		
	def writeVector(self, file, v):
		# Write length of vector
		file.write(struct.pack("L", len(v)))
		
		# Write vector elements
		for x in v:
			x.write(file)
			
			
	def writeArray(self, file, a, a_num_elems):
		file.write(struct.pack("L", a_num_elems))
		a.tofile(file)
		
	def writeToDisk(self, path):
		f = file(path, 'wb')
		
		MAGIC_NUMBER = 5456751
		FORMAT_VERSION = 1
		
		f.write(struct.pack("L", MAGIC_NUMBER))
		f.write(struct.pack("L", FORMAT_VERSION))
		f.write(struct.pack("L", self.num_uv_mappings))
		
		self.writeVector(f, self.used_materials)
		self.writeVector(f, self.uv_set_expositions)
		
		self.writeArray(f, self.vert_positions, len(self.vert_positions) / 3)
		self.writeArray(f, self.vert_normals, len(self.vert_normals) / 3)
		self.writeArray(f, self.uv_pairs, len(self.uv_pairs) / 2)
		self.writeArray(f, self.triangles, len(self.triangles) / 7)
		
		f.close


def makeSafeFilename(s):
	regexp = re.compile(r'\W') # \W = [^a-zA-Z0-9_]
	return regexp.sub("_", s)
	
def test():


	print(makeSafeFilename("C:\pfdgfdg/dfgdfg/(^*&^*&^&*.jph"))

	N = 10000

	####################################
	start_time = time.time()

	m = OldIndigoMesh()
	m.num_uv_mappings = 1
	m.used_materials.append(IndigoMeshString("mat1"))
	m.uv_set_expositions.append(UVSetExposition(IndigoMeshString("default"), 0))

	for i in range(N):
		m.vert_positions.append(Vec3f(0,1,2))
	for i in range(N):
		m.vert_normals.append(Vec3f(0,0,1))
	for i in range(N):
		m.uv_pairs.append(Vec2f(0,0))

	for i in range(N):
		m.triangles.append(Triangle([i, i, i], [i, i, i], 0))

	m.writeToDisk("test.igmesh")

	duration = time.time() - start_time

	print("OldIndigoMesh: " + str(duration) + " s")

	########################################
	start_time = time.time()

	m = IndigoMesh()
	m.num_uv_mappings = 1
	m.used_materials.append(IndigoMeshString("mat1"))
	m.uv_set_expositions.append(UVSetExposition(IndigoMeshString("default"), 0))

	for i in range(N):
		m.addVertPosition(0, 1, 2)
		#m.vert_positions.append(0)
		#m.vert_positions.append(1)
		#m.vert_positions.append(2)
	for i in range(N):
		m.addVertNormal(0, 1, 2)
		#m.vert_normals.append(0)
		#m.vert_normals.append(1)
		#m.vert_normals.append(2)
	for i in range(N):
		m.addUVPair(0, 0)
		#m.uv_pairs.append(0)
		#m.uv_pairs.append(0)

	for i in range(N):
		m.addTriangle(i,i,i,i,i,i,0)
		#m.triangles.append(i)
		#m.triangles.append(i)
		#m.triangles.append(i)
		#m.triangles.append(i)
		#m.triangles.append(i)
		#m.triangles.append(i)
		#m.triangles.append(0)

	m.writeToDisk("test2.igmesh")

	duration = time.time() - start_time

	print("IndigoMesh: " + str(duration) + " s")

	########################################

#test()
